var _ = require('lodash');
var engine = require('ace/engine');
var canvasUtils = require('ace/canvasUtils');
var graphics = require('ace/graphics');
var mkrunloop = require('ace/loop');
var controls = require('ace/controls');


module.exports = function(spec) {

  spec = spec || {};
  _.defaults(spec, {
    scale: 1,
    size: [750, 500]
  });

  var canvas = canvasUtils.makeCanvas({
    width: spec.size[0] * spec.scale,
    height: spec.size[1] * spec.scale
  });

  document.body.appendChild(canvas);

  var buffer;
  if (spec.scale == 1) {
    buffer = canvas;
  }
  else {
    buffer = canvasUtils.makeCanvas({
      width: spec.size[0],
      height: spec.size[1]
    });
  }

  var ng = engine(
    _.assign({
      render: {canvas: buffer}
    }, spec)
  );

  if (spec.scale != 1) {
    ng.signals.frame.watch(function() {
      var context = canvas.getContext('2d');
      context.save();
      context.scale(spec.scale, spec.scale);
      context.drawImage(buffer, 0, 0);
      context.restore();
    });
  }

  var loop = mkrunloop({
    update: ng.update,
    frame: ng.frame
  });


  window.addEventListener('blur', loop.stop);
  window.addEventListener('focus', loop.start);
  loop.start();

  return {
    engine: ng,
    loop: loop,
    buffer: buffer,
    canvas: canvas
  };

};

function toggler() {
  var i = 1;
  return function() {
    i = (i + 1) % 2;
    return !!i;
  };
}
