var _ = require('lodash');

exports.font = function(image, charSize, charmap) {

  charmap = charmap || {};

  var font = {};

  var charsWide = Math.floor(image.width / charSize[0]);

  font.text = function(text) {

    var dx = 0, dy = 0;
    var drawArgs = [];

    _.times(text.length, function(i) {
      var code = text.charCodeAt(i);
      if (charmap[code] !== undefined)
        code = charmap[code];
      var x = code % charsWide;
      var y = Math.floor(code / charsWide);
      if (code == 10) {
        dx = 0;
        dy += charSize[1];
      }
      else {
        drawArgs.push([
            image,
            x * charSize[0],
            y * charSize[1],
            charSize[0],
            charSize[1],
            dx,
            dy,
            charSize[0],
            charSize[1]
        ]);
        dx += charSize[0];
      }
    });

    return {
      draw: function(view) {
        view.withContext(function(c) {
          drawArgs.forEach(function(xs) {
            c.drawImage.apply(c, xs);
          });
        });
      }
    };
  };

  return font;

};

function layout(text, lineWidth) {
  var lines = text.split('\n');
  var out = [];
  lines.forEach(function(line) {
    var words = line.split(' ');
    var x = 0;
    words.forEach(function(word) {
      if (x + word.length > lineWidth) {
        if (word.length > lineWidth) {
          out.push(word.substr(0, lineWidth + 1));
          word = word.substr(lineWidth + 1);
        }
        out.push('\n');
        x = 0;
      }
      out.push(word);
      out.push(' ');
      x += word.length + 1;
    });
    out.push('\n');
    x = 0;
  });
  return out.join('').replace(/ \n/g,'\n');
};


exports.textBox = function(font, sizeInCells, text) {
  text = layout(text, sizeInCells[0]);
  return font.text(text);
};

exports.layout = layout;
