var _ = require('lodash');

exports.makeCanvas = function(spec) {

  /** Creates and returns a <canvas> element with given
      size. Sets image smoothing off. */

  spec = _.defaults(spec || {}, {
    width: 750,
    height: 500
  });

  var canvas = document.createElement('canvas');

  canvas.width = spec.width;
  canvas.height = spec.height;

  canvas.setAttribute('style', 'image-rendering: pixelated; image-rendering: -webkit-crisp-edges');

  var context = canvas.getContext('2d');
  context.imageSmoothingEnabled = false;
  context.webkitImageSmoothingEnabled = false;
  context.mozImageSmoothingEnabled = false;

  return canvas;
};

exports.makeStretchyCanvas = function(spec) {

  /** As makeCanvas(), except returned canvas has update() which
      stretches it to the window size. */

  spec = _.defaults(spec || {}, {
    width: 750,
    height: 500
  });

  var canvas = exports.makeCanvas();

  canvas.style.display = 'block';
  canvas.style.margin = 'auto';

  var aspect = spec.width / spec.height;

  function update() {
    var bounds = {height: window.innerHeight, width: window.innerWidth};

    if (bounds.width / bounds.height < aspect) {
      canvas.style.width = bounds.width + 'px';
      canvas.style.height = Math.floor(bounds.width / aspect) + 'px';
    }
    else {
      canvas.style.height = bounds.height + 'px';
      canvas.style.width = Math.floor(bounds.height * aspect) + 'px';
    }

    canvas.height = spec.height;
    canvas.width = spec.width;
  }

  update();

  canvas.update = update;

  return canvas;
};

exports.clear = function(context, col) {
  if (col) {
    context.fillStyle = col;
    context.fillRect(0, 0, context.canvas.width, context.canvas.height);
  }
  else
    context.clearRect(0, 0, context.canvas.width, context.canvas.height);
};
