var _ = require('lodash');
var frp = require('frp');

module.exports = function(spec) {

  spec = spec || {};

  var ng = {};

  var systems = [];

  var updateCell = frp.cell();
  var frameCell = frp.cell();

  ng.signals = {
    update: updateCell.signal,
    frame: frameCell.signal
  };

  function syscall(name) {
    var args = _.toArray(arguments).slice(1);
    _.forEach(systems, function(sys) {
      (sys[name] || _.identity).apply(null, args);
    });
  }

  ng.update = _.partial(syscall, 'update');
  ng.frame = _.partial(syscall, 'frame');

  systems.push({
    update: updateCell.set,
    frame: frameCell.set
  });

  function init(name, mod) {
    var sys = mod(ng, spec[name] || {});
    systems.push(sys);
    if (!ng[name])
      ng[name] = sys;
  }

  if (spec.render !== false)
    init('render', require('./systems/render'));

  if (spec.physics !== false)
    init('physics', require('./systems/physics'));

  return ng;

};
