var utils = require('./utils');

exports.tilemapGraphic = function(gl, tilemap, tilesetImage) {

  var graphic = {};

  var tilesetTexture = utils.createTexture(gl, tilesetImage);
  var vbuf = utils.createQuadVerts(gl);

  var prog = utils.linkProgram(
    gl,
    `
      precision mediump float;
      attribute vec2 a_vertexPosition;

      void main(void) {
        gl_Position = vec4(a_vertexPosition, 0.0, 1.0);
      }
    `,
    `
      precision mediump float;

      uniform sampler2D u_tileset;
      uniform vec2 u_tilesetSize;

      void main() {
        vec2 samplerCoord = gl_FragCoord.xy / u_tilesetSize;
        gl_FragColor = texture2D(u_tileset, samplerCoord);
      }
    `
  );

  var u_tileset = gl.getUniformLocation(prog, 'u_tileset');
  var u_tilesetSize = gl.getUniformLocation(prog, 'u_tilesetSize');

  var layer = tilemap.layers[0];

  graphic.draw = function(gl) {

    gl.useProgram(prog);

    gl.activeTexture(gl.TEXTURE0);
    gl.bindTexture(gl.TEXTURE_2D, tilesetTexture);
    gl.uniform1i(u_tileset, 0);
    gl.uniform2f(u_tilesetSize, tilesetImage.width, tilesetImage.height);

    gl.enableVertexAttribArray(0);
    gl.vertexAttribPointer(0, 2, gl.FLOAT, false, 0, 0);
    gl.bindBuffer(gl.ARRAY_BUFFER, vbuf);
    gl.drawArrays(gl.TRIANGLES, 0, 6);
  };

  return graphic;
};
