var _ = require('lodash');

exports.compileShader = function(gl, source, type) {

  if (typeof(type) == 'string')
    type = {
      'vertex': gl.VERTEX_SHADER,
      'fragment': gl.FRAGMENT_SHADER
    }[type];

  var shader = gl.createShader(type);
  gl.shaderSource(shader, source);
  gl.compileShader(shader);
  if (!gl.getShaderParameter(shader, gl.COMPILE_STATUS)) {
    throw gl.getShaderInfoLog(shader);
  }
  return shader;
};

exports.linkProgram = function(gl, vertex, fragment) {

  if (typeof(vertex) == 'string')
    vertex = exports.compileShader(gl, vertex, 'vertex');

  if (typeof(fragment) == 'string')
    fragment = exports.compileShader(gl, fragment, 'fragment');

  var prog = gl.createProgram();
  gl.attachShader(prog, vertex);
  gl.attachShader(prog, fragment);
  gl.linkProgram(prog);

  if (!gl.getProgramParameter(prog, gl.LINK_STATUS)) {
    throw gl.getProgramInfoLog(prog);
  }

  return prog;
};

exports.createTexture = function(gl, data) {
  var tex = gl.createTexture();
  var t2d = gl.TEXTURE_2D;
  gl.bindTexture(t2d, tex);
  gl.texImage2D(t2d, 0, gl.RGBA, gl.RGBA, gl.UNSIGNED_BYTE, data);
  gl.texParameteri(t2d, gl.TEXTURE_MAG_FILTER, gl.NEAREST);
  gl.texParameteri(t2d, gl.TEXTURE_MIN_FILTER, gl.NEAREST);
  gl.texParameteri(t2d, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);
  gl.texParameteri(t2d, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);
  return tex;
};

exports.createQuadVerts = function(gl) {
  var vbuf = gl.createBuffer();
  gl.bindBuffer(gl.ARRAY_BUFFER, vbuf);
  gl.bufferData(
    gl.ARRAY_BUFFER,
    new Float32Array([-1, -1, 1, -1, -1, 1, 1, -1, -1, 1, 1, 1]),
    gl.STATIC_DRAW
  );
  return vbuf;
};


exports.context = function(size) {

  var canvas = document.createElement('canvas');
  canvas.width = size[0];
  canvas.height = size[1];
  canvas.style.width = size[0] + 'px';
  canvas.style.height = size[1] + 'px';
  canvas.style.imageRendering = 'pixelated';
  var gl = canvas.getContext('webgl') || canvas.getContext('experimental-webgl');

  //gl.viewport(0, 0, size[0], size[1]);
  gl.pixelStorei(gl.UNPACK_FLIP_Y_WEBGL, true);

  return gl;

};
