var _ = require('lodash');

module.exports = function runloop(spec) {

  var timeStep = spec.timeStep || 1/60;
  var update = spec.update || _.identity;
  var frame = spec.frame || _.identity;

  var running = false;
  var stopped = true;

  function start() {

    if (!stopped)
      return;

    running = true;
    stopped = false;

    window.requestAnimationFrame(function(start) {

      var acc = 0;

      function doFrame(time) {
        var dt = (time - start) / 1000;
        start = time;

        acc += dt;
        while (acc >= timeStep) {
          update(timeStep);
          acc -= timeStep;
        }
        frame(dt);

        nextFrame();
      }

      function nextFrame() {
        if (running) {
          window.requestAnimationFrame(doFrame);
        }
        else
          stopped = true;
      }

      nextFrame();

    });
  }

  function stop() {
    running = false;
  }

  return {start: start, stop: stop};
};
