var _ = require('lodash');
var frp = require('frp');
var p2 = require('p2');
var util = require('../util');
var T = require('../transform');

module.exports = function(ng, spec) {

  var sys = {};

  var nodes = [];

  var world = new p2.World({
    gravity: spec.gravity || [0, 400]
  });

  var collisionPipe = frp.pipe();
  sys.collision = collisionPipe.event;

  sys.world = world;

  sys.node = function(spec) {

    var node = {};

    _.defaults(spec, {
      mass: 1,
      pos: [0, 0],
      angle: 0,
      force: [0, 0],
      velocity: [0, 0]
    });

    var pipe = frp.pipe();
    node._pipe = pipe;
    node.event = pipe.event;

    node.body = new p2.Body({
      mass: spec.mass,
      position: spec.pos,
      angle: spec.angle,
      velocity: spec.velocity
    });

    node.body.aceNode = node;

    node.force = spec.force;

    world.addBody(node.body);

    node.transform = frp.cell(T.pos(spec.pos));
    node.position = node.transform.map('pos');
    node.angle = node.transform.map('rot');

    nodes.push(node);

    return node;
  };

  world.on('beginContact', onContactEvent);
  world.on('endContact', onContactEvent);

  function onContactEvent(e) {
    var nodeA = e.bodyA.aceNode;
    var nodeB = e.bodyB.aceNode;

    [nodeA, nodeB].filter(_.identity).forEach(function(node) {
      node._pipe.fire(wrapEvent(e, node));
    });

    if (nodeA && nodeB)
      collisionPipe.fire(wrapEvent(e));
  }

  sys.update = function(dt) {

    util.removeDead(nodes, function(node) {
      world.removeBody(node.body);
    });

    _.forEach(nodes, function(node) {
      node.body.force[0] = node.force[0];
      node.body.force[1] = node.force[1];
    });

    world.step(dt);

    _.forEach(nodes, function(node) {
      node.transform.set(T.prs(
        node.body.position,
        node.body.angle,
        [1, 1]
      ));
    });
  };

  sys.debugGraphic = function() {
    var dd = require('../debugDraw');
    return {
      draw: function(view) {
        view.withContext(function(c) {
          dd.world(c, world);
        });
      }
    };
  };

  ng.physics = sys;

  return sys;
};

function wrapEvent(e, thisNode) {
  var r = _.assign({}, e, {
    original: e,
    nodeA: e.bodyA.aceNode,
    nodeB: e.bodyB.aceNode
  });

  if (thisNode) {
    r.thisNode = thisNode;
    r.otherNode = (r.nodeA === thisNode ? r.nodeB : r.nodeA);

    r.thisShape = (r.nodeA === thisNode ? r.shapeA : r.shapeB);
    r.otherShape = (r.nodeA === thisNode ? r.shapeB : r.shapeA);
  }

  return r;
}
