var ace = require('ace');
var frp = require('frp');
var util = require('ace/util');
var _ = require('lodash');
var V = require('mxd').Vector;

exports.tmx = function(tilesetImage, tmx) {
  var parser = new DOMParser();
  var doc = parser.parseFromString(tmx, 'text/xml');

  var tileset = doc.getElementsByTagName('tileset')[0];
  var tw = parseInt(tileset.getAttribute('tilewidth'));
  var th = parseInt(tileset.getAttribute('tileheight'));
  var firstgid = parseInt(tileset.getAttribute('firstgid'));

  var layers = _.toArray(doc.getElementsByTagName('layer')).map(function(layer) {
    return {
      width: parseInt(layer.getAttribute('width')),
      height: parseInt(layer.getAttribute('height')),
      data: (
        layer
        .getElementsByTagName('data')[0]
        .textContent
        .replace(/\s/g, '')
        .split(',')
        .map(function(x) { return parseInt(x) - firstgid; })
      )
    };
  });

  var firstGid = 1; // FIXME
  var tiles = _.toArray(tileset.getElementsByTagName('tile')).map(function(elem) {
    var propElems = elem.getElementsByTagName('properties')[0].getElementsByTagName('property');
    var tile = _.object(
      _.toArray(propElems).map(function(p) {
        return [p.getAttribute('name'), p.getAttribute('value')];
      })
    );
    tile.id = parseInt(elem.getAttribute('id'));
    tile.id = tile.id + firstGid;
    console.log(tile);
    return tile;
  });
  tiles = _.indexBy(tiles, 'id');

  var objectGroups = _.toArray(doc.getElementsByTagName('objectgroup')).map(function(group) {
    return _.toArray(group.getElementsByTagName('object')).map(function(object) {

      var propElems = object.getElementsByTagName('property');
      var r = _.object(
        _.toArray(propElems).map(function(p) {
          return [p.getAttribute('name'), p.getAttribute('value')];
        })
      );

      r.name = object.getAttribute('name');
      r.pos = [parseFloat(object.getAttribute('x')),
        parseFloat(object.getAttribute('y'))];
      r.tile = tiles[object.getAttribute('gid')];
      return r;

    });
  });


  return exports.tilemap(tw, th, tilesetImage, tiles, layers, objectGroups);
};

exports.tilemap = function(tw, th, tilesetImage, tiles, layers, objectGroups) {
  var map = {};

  map.tiles = tiles;
  map.layers = layers;

  map.objectGroups = objectGroups;
  map.objects = _.flatten(objectGroups);

  var tsw = tilesetImage.width / tw;

  map.get = function(xy, layerNo) {
    var id = map.getID(xy, layerNo);
    return tiles[id + 1] || {id:id}; // fix
  };

  map.getID = function(xy, layerNo) {
    var layer = layers[layerNo];

    return layer.data[xy[1] * layer.width + xy[0]];
  };

  map.set = function(xy, layerNo, tileNo) {
    var layer = layers[layerNo];
    layer.data[xy[1] * layer.width + xy[0]] = tileNo;
  };

  map.getGraphic = function(layerNumbers) {
    return {
      draw: function(view) {

        // project cam bounds to world space
        var vsz = view.getViewport();
        var screenToWorld = view.getTransform().invert();
        var corners = [
          screenToWorld.apply([0,      0]),
          screenToWorld.apply([vsz[0], 0]),
          screenToWorld.apply([vsz[0], vsz[1]]),
          screenToWorld.apply([0,      vsz[1]]),
        ];

        // project again to grid space
        corners = corners.map(function(p) {
          return V.floor(V.div(p, [tw, th]));
        });

        // draw all cells in grid within min AABB of projected
        // camera rect
        var xs = util.minMax(_.map(corners, '0'));
        var ys = util.minMax(_.map(corners, '1'));

        var startCoord = [xs.min, ys.min];
        var endCoord = [xs.max + 1, ys.max + 1];

        view.withContext(function(c) {

          layerNumbers.forEach(function(layerNumber) {

            var layer = layers[layerNumber];

            V.iterect(startCoord, endCoord, function(gridPos) {

              if (!(0 <= gridPos[0] && gridPos[0] < layer.width)) return;
              if (!(0 <= gridPos[1] && gridPos[1] < layer.height)) return;

              var tileno = layer.data[gridPos[1] * layer.width + gridPos[0]];
              var tgx = tileno % tsw;
              var tgy = Math.floor(tileno / tsw);
              c.drawImage(
                tilesetImage,
                tw * tgx,
                th * tgy,
                tw,
                th,
                tw * gridPos[0],
                th * gridPos[1],
                tw,
                th
              );
            });

          });
        });
      }
    };
  };

  map.graphic = map.getGraphic(_.range(layers.length));

  return map;
};
