var _ = require('lodash');
var frp = require('frp');

var time = module.exports;


/*
  Returns an event that will fire the result of 'getVal' after 'dur' seconds.
*/
time.wait = function(ng, dur, getVal) {
  var r = frp.pipe();

  var a = 0;

  var cb = function(dt) {
    a += dt;
    if (a >= dur) {
      ng.signals.update.unwatch(cb);
      var val = (getVal || _.noop)();
      r.fire(val);
    }
  };

  ng.signals.update.watch(cb);

  return r.event;
};


/*
  Returns an event that will call 'getVal' every 'dur' seconds, firing
  its return value each time.
*/
time.intervals = function(ng, durs, getVal) {

  var r = frp.pipe();

  var i = 0;

  function iter() {

    var dur = durs[i];
    i = (i + 1) % durs.length;

    time.wait(ng, dur, function() {
      r.fire((getVal || _.noop)());
      iter();
    });
  }

  iter();

  return r.event;
};


/*
  Returns an event that will fire all values from 'evt' delayed
  by 'dur' seconds.
*/
time.delay = function(ng, evt, dur) {

  var r = frp.pipe();

  evt.watch(function(x) {
    time.wait(ng, dur, x).watch(r.fire);
  });

  return r.event;

};
