/* Best transform as of 2015-04-29 */

var _ = require('lodash');

var M = require('mxd/Matrix');
var V = require('mxd/Vector');
var R = require('mxd/Rect');

var T = module.exports = {};

function Transform(pos, rot, scale) {
  this.pos = pos;
  this.rot = rot;
  this.scale = scale;

  Object.defineProperties(this, {
    matrix: {get: _.once(function() {
      var m = M.mul(M.scaling(scale), M.rotation(rot));
      m[0][2] = pos[0];
      m[1][2] = pos[1];
      m[2] = [0, 0, 1];
      return m;
    })}
  });
}

T.prs = function(pos, rot, scale) {
  return new Transform(pos, rot, scale);
};

T.identity = T.prs([0, 0], 0, [1, 1]);

T.pos = function(v) {
  return T.prs(v, 0, [1, 1]);
};

T.rot = function(a) {
  return T.prs([0, 0], a, [1, 1]);
};

T.scale = function(v) {
  return {
    pos: [0, 0],
    rot: 0,
    scale: v,
    matrix: M.scaling(v)
  };
};

T.mul = function(a, b) {
  return T.prs(
    T.apply(b, a.pos),
    a.rot + b.rot,
    V.mul(a.scale, b.scale)
  );
};

/*
  Instance methods support 2 forms:
    1. T.apply(t, v)
    2. t.apply(v)

  (1) is marginally faster.
*/
function def(name, impl) {
  T[name] = impl;
  Transform.prototype[name] = function() {
    var args = _.toArray(arguments);
    args.unshift(this);
    return impl.apply(null, args);
  };
}

def('copy', function(x, y) {
  var z = _.assign({}, x, y);
  return T.prs(z.pos, z.rot, z.scale);
});

def('apply', function(t, v) {
  //return M.apply(t.matrix, [v[0], v[1], 1]).slice(0, 2);
  return V.add(t.pos, V.rotate(V.mul(v, t.scale), t.rot));
});

def('neg', function(t) {
  return T.prs(
    V.invert(t.pos),
    -t.rot,
    V.scaleInverse(t.scale, 1)
  );
});

def('translate', function(t, v) {
  return T.copy(t, {pos: V.add(t.pos, v)});
});

def('invert', function(t) {
  return T.prs(
    V.rotate(V.div(V.invert(t.pos), t.scale), -t.rot),
    -t.rot,
    V.scaleInverse(t.scale, 1)
  );
});

def('toContext', function(t) {
  var m = t.matrix;
  return [m[0][0], m[1][0], m[0][1], m[1][1], t.pos[0], t.pos[1]];
});

def('applyToContext', function(t, context) {
  context.transform.apply(context, T.toContext(t));
});
