var _ = require('lodash');
var V = require('mxd').Vector;
var frp = require('frp');

var util = module.exports;


function sqrt0(x) { 
  return x === 0 ? 0 : Math.sqrt(x);
}

util.findJumpVel = function(jumpHeight, gravity, mass) {
  return V.invert(V.scale(gravity, jumpHeight * 2).map(sqrt0));
};

util.set = function(obj, name, x) {
  obj[name] = x;
};

util.get = function(obj, name) {
  return obj[name];
};

util.setter = function(obj, name) {
  return _.partial(util.set, obj, name);
};

util.getter = function(obj) {
  return _.partial(util.get, obj);
};

util.removeDead = function(xs, dtor) {

  var dead = [];
  _.forEach(xs, function(node) {
    if (node.dead)
      dead.push(node);
  });

  dead.forEach(function(node) {
    (dtor || _.noop)(node);
    _.pull(xs, node);
  });
};


util.otherNodeFromShape = function(e, shape) {
  var otherBody;
  if (e.shapeA === shape) otherBody = e.bodyB;
  if (e.shapeB === shape) otherBody = e.bodyA;
  return otherBody && otherBody.aceNode;
};

util.shapeContactsFromEvent = function(ng, event, shape) {

  var nearbyNodes = {};
  var r = frp.cell();

  function update() {
    r.set(_.values(nearbyNodes));
  }

  event.bind(function(e) {
    var otherNode = util.otherNodeFromShape(e, shape);
    if (otherNode) {
      if (e.type == 'beginContact') {
        nearbyNodes[otherNode.body.id] = otherNode;
        update();
      }
      if (e.type == 'endContact') {
        delete nearbyNodes[otherNode.body.id];
        update();
      }
    }
  });

  return r.signal;
};

util.shapeContacts = function(ng, pnode, shape) {
  return util.shapeContactsFromEvent(ng, pnode.event, shape);
};

util.anim = function(ng, frames, dur) {
  var timer = ng.signals.update.sum();
  var framen = timer.map(function(x) {
    return Math.floor((x / dur) % frames.length);
  });
  return framen.map(function(i) { return frames[i]; });
};

util.timer = function(ng, seconds) {
  var r = frp.pipe();

  var update = ng.signals.update.ref();

  var positive = function(x) { return x > 0; };

  var t = update.sum();
  var diff = t.map(function(x) { return x - seconds; });

  diff.filter(positive).watch(function(x) {
    update.release();
    r.fire(x);
  });

  return r.event;
};

util.issig = function(x) {
  return !!(x.bind && x.unbind);
};

util.tosig = function(x) {
  if (util.issig(x))
    return x;
  if (x !== undefined)
    return frp.signal.constant(x);
};

util.sigargs = function(fn) {
  return function() {
    var args = _.toArray(arguments);
    var sigs = _.map(args, util.tosig);
    return fn.apply(null, sigs);
  };
};

util.mod = function(a, b) {
  if (a < 0)
    return b + (a % b);
  return a % b;
};

util.minMax = function(xs) {
  var x0, x1;
  xs.forEach(function(x) {
    if (x0 === undefined)
      x0 = x1 = x;
    else if (x < x0)
      x0 = x;
    else if (x > x1)
      x1 = x;
  });
  return {min: x0, max: x1};
};
