var V = require('mxd/Vector');
var T = require('./transform');

function View(context, tr, alpha, comp) {
  this._context = context;
  this._tr = tr || T.identity;
  this._alpha = (alpha === undefined) ? 1 : alpha;
  this._comp = comp || 'source-over';
}

View.prototype.transform = function(newTr) {
  return new View(this._context, T.mul(newTr, this._tr), this._alpha, this._comp);
};

View.prototype.withTransform = function(newTr) {
  return new View(this._context, newTr, this._alpha, this._comp);
};

View.prototype.withAlpha = function(a) {
  return new View(this._context, this._tr, a, this._comp);
};

View.prototype.withComp = function(comp) {
  return new View(this._context, this._tr, this._alpha, comp);
};

View.prototype.getTransform = function() {
  return this._tr;
};

View.prototype.getViewport = function() {
  var can = this._context.canvas;
  return [can.width, can.height];
};

View.prototype.withContext = function(cb) {
  var context = this._context;
  context.save();
  context.globalAlpha = this._alpha;
  context.globalCompositeOperation = this._comp;

  var tr1 = T.copy(this._tr, {
    pos: V.floor(this._tr.pos)
  });

  T.applyToContext(tr1, context);
  cb(context);
  context.restore();

};

module.exports = function makeView(context, tr) {
  return new View(context, tr);
};
