var ace = require('ace');
var ld34utils = require('./ld34utils');
var assets = require('./assets');
var R = require('mxd').Rect;
var p2 = require('p2');
var T = require('ace/transform');
var V = require('mxd').Vector;


function tileRect(gx, gy, w, h, offset) {
  return ace.graphics.image(
    assets.tiles,
    offset || [16, 16],
    R.fromSize([32 * gx, 32 * gy], [32 * w, 32 * h])
  );
}

exports.fight = function(env) {
  var pos = env.markers.bossSpawn;

  var boss = {};

  var pnode = env.ng.physics.node({
    pos: pos,
    mass: 100
  });

  var w = 32 * 3 - 48;
  var h = 32 * 5 - 16;
  var s = new p2.Box({width: w, height: h});
  pnode.body.addShape(s);

  boss.graphic = tileRect(0, 23, 3, 5);
  boss.graphic = ace.graphics.transform(
    boss.graphic,
    pnode.position.map(function(x) { return V.add(x, [-w/2, -h/2]); }).map(T.pos)
  );

  env.objects.push(boss);
  env.ng.signals.update.watch(function(dt) {
    process(env, dt);
  });

  pnode.object = boss;
  boss.type = 'boss';

  boss.die = function() {
    ld34utils.explode(env, pnode.position.value, 100, 20, ace.graphics.point());//tileRect(6*0, 19*0, 1, 1));
    env.objects.pull(boss);
    pnode.dead = true;
    setTimeout(function() {
      env.ng.render.graphic = ace.graphics.image(assets.win, [0, 0]);
    }, 1000);
  };

  shoot(env);
};

function process(env, dt) {
}

var shoot = function(env) {

  var pos = V.add(env.markers.bossSpawn, [100, 0]);

  var shot = {};

  var pnode = env.ng.physics.node({
    pos: pos,
    mass: 100,
    velocity: [128, 128]
  });

  var s = new p2.Circle({radius: 32});
  pnode.body.addShape(s);
  pnode.body.gravityScale = 0;

  var shotMat = new p2.Material();
  s.material = shotMat;

  var conMat = new p2.ContactMaterial(shotMat, assets.envMat, {
    friction: 0,
    restitution: 1
  });
  env.ng.physics.world.addContactMaterial(conMat);

  var frames = [22, 24, 26].map(function(y) {
    return tileRect(6, y, 2, 2, [32, 32]);
  });

  var kframes = [[6, 20], [6, 21], [7, 20], [7, 21]].map(function(xy) {
    return tileRect(xy[0], xy[1], 1, 1, [16, 16]);
  });

  var ographic = ace.util.anim(env.ng, frames, .05);
  var kgraphic = ace.util.anim(env.ng, kframes, .05);

  var graphic = ace.graphics.list([ographic, kgraphic]);

  shot.graphic = ace.graphics.transform(
    graphic,
    pnode.position.map(T.pos)
  );

  shot.die = function() {
    env.objects.pull(shot);
    pnode.dead = true;
  };

  pnode.event.watch(function(e) {
    if (e.otherNode.object && e.otherNode.object.type == 'boss') {
      shot.die();
      e.otherNode.object.die();
    }
  });

  env.objects.push(shot);
};
