var ace = require('ace');
var _ = require('lodash');
var V = require('mxd').Vector;

var growAnimDuration = .25;
var shrinkAnimDuration = .25;
var growDuration = 3;


module.exports = function grow(object) {

  if (!object.growScale)
    return;

  if (object.growState)
    return;

  object.growState = 'growing';
  object.growTTL = growAnimDuration;
  (object.onGrowStart || _.noop)();
};

module.exports.overlayGraphic = function(object) {
  return {
    draw: function(view) {
      if (object.growState == 'big') {

        var dur = object.growDuration || growDuration;

        var factor = (object.growTTL / dur);

        var x = 16, y = 16;
        var twopi = 2 * Math.PI;
        var phase = -.25;

        var start = phase * twopi;
        var end = ((phase - factor) % 1) * twopi;

        view.withContext(function(c) {
          c.fillStyle = '#2df';
          c.beginPath();
          c.arc(x, y, 8, 0, twopi, false);
          c.fill();

          c.fillStyle = '#888';
          c.beginPath();
          c.moveTo(x, y);
          c.arc(x, y, 6, start, end, false);
          c.fill();
        });
      }
    },
    bounds: function(view) {
    }
  };
};

function animate(object, factor) {
  object.growScale.set(1 + factor);

  if (object.growOffsets) {
    object.growOffsets.forEach(function(offsets, i) {

      var offset = V.add(
        offsets[0],
        V.scale(
          V.sub(offsets[1], offsets[0]),
          factor
        )
      );
      object.pnode.body.shapes[i].position = offset;
    });
    object.pnode.body.updateAABB();
    object.pnode.body.updateBoundingRadius();
  }
}

module.exports.update = function(env, dt) {

  env.objects.items.value.forEach(function(object) {

    if (object.growState == 'growing') {

      if (object.growTTL <= 0) {
        object.growState = 'big';
        object.growTTL = object.growDuration || growDuration;
        (object.onGrowEnd || _.noop)();
      }
      else {
        object.growTTL -= dt;
        animate(object, 1 - (object.growTTL / growAnimDuration));
      }
    }

    else if (object.growState == 'big') {
      if (object.growTTL <= 0) {
        object.growState = 'shrinking';
        object.growTTL = shrinkAnimDuration;
        (object.onShrinkStart || _.noop)();
      }
      else {
        object.growTTL -= dt;
      }
    }

    else if (object.growState == 'shrinking') {
      if (object.growTTL <= 0) {
        object.growState = null;
        (object.onShrinkEnd || _.noop)();
      }
      else {
        object.growTTL -= dt;
        animate(object, object.growTTL / shrinkAnimDuration);
      }
    }

  });
};
