var ace = require('ace');
var assets = require('./assets');
var R = require('mxd').Rect;
var V = require('mxd').Vector;
var _ = require('lodash');
var T = require('ace/transform');

exports.tileGraphic = function(tileID) {
  var firstGid = 1;
  var tilesPerRow = 8;
  var i = tileID - firstGid;
  var tx = 32 * (i % tilesPerRow);
  var ty = 32 * Math.floor(i / tilesPerRow);

  return ace.graphics.image(
    assets.tiles,
    [16, 16],
    R.fromSize([tx, ty], [32, 32])
  );
};


exports.explode = function(env, pos, radius, n, graphic) {

  var obj = {};

  obj.ttl = 1;

  env.objects.push(obj);

  pos = _.toArray(pos);

  var particles = [];
  _.times(n, function() {
    var p = {};

    var a = Math.random(Math.PI * 2);
    var dir = [Math.sin(a), Math.cos(a)];
    var vel = V.scale(dir, Math.random() * 100);

    p.pos = pos;
    p.vel = vel;

    particles.push(p);
  });

  obj.process = function(dt) {
    particles.forEach(function(p) {
      p.pos = V.add(p.pos, V.scale(p.vel, dt));
    });
  };

  obj.die = function() {
    env.objects.pull(obj);
  };

  obj.graphic = {
    bounds: function() {},
    draw: function(view) {
      view.withContext(function(c) {
        particles.forEach(function(p) {
          var nv = view.transform(T.pos(V.scale(p.pos, 1)));
          graphic.draw(nv);
        });
      });
    }
  };
};
