var V = require('mxd').Vector;
var p2 = require('p2');
var assets = require('./assets');

exports.addMapCollision = function(tilemap, ng) {
  var tileSize = [32, 32];

  function isSolid(tile) {
    return tile.solid;
  }

  // Map Bodies
  var layer = tilemap.layers[0];

  var runLoc, runLen;

  function endRun() {
    if (runLoc !== null) {

      var pos = V.mul(V.add(runLoc, [runLen / 2, .5]), tileSize);

      var pnode = ng.physics.node({
        pos: pos,
        mass: 0
      });

      var s = new p2.Box({width:runLen * 32, height:32});
      s.material = assets.envMat;
      pnode.body.addShape(s);
    }
    runLoc = null;
    runLen = null;
  }

  function solidAt(loc) {
    for (var i in tilemap.layers) {
      var tile = tilemap.get(loc, i);
      if (isSolid(tile))
        return true;
    }
    return false;
  }

  for (var y=0; y<layer.height; y++) {
    for (var x=0; x<layer.width; x++) {
      var loc = [x, y];

      var solid = solidAt(loc);
      var solidAbove = solidAt(V.add(loc, [0, -1]));
      var solidLeft = solidAt(V.add(loc, [-1, 0]));
      var solidRight = solidAt(V.add(loc, [1, 0]));
      var solidBelow = solidAt(V.add(loc, [0, 1]));

      if (solid && (!solidAbove || !solidLeft || !solidRight || !solidBelow)) {

        if (runLoc == null) {
          runLoc = loc;
          runLen = 1;
        }
        else {
          runLen += 1;
        }

      }
      else
        endRun();
    }
    endRun();
  }
  endRun();

};
