
var platformer = require('ace/platformer');
var graphics = require('ace/graphics');
var assets = require('./assets');
var T = require('ace/transform');
var V = require('mxd').Vector;
var R = require('mxd').Rect;
var p2 = require('p2');
var makeDpad = require('ace/dpad');
var controls = require('ace/controls');
var util = require('ace/util');
var frp = require('frp');
var _ = require('lodash');

var grow = require('./grow');


module.exports = function(ng, pos) {

  var mass = 1;
  var jumpHeight = 80;
  var walkSpeed = 200;
  var airControl = 350;

  var player = {};

  var pnode, pfBody, jumpVel;

  var posCell = frp.cell();

  // Dpad
  var dpad = makeDpad();
  dpad.map('0').bind(function(intent) {
    if (pfBody)
      pfBody.setIntent(intent);
  });

  function initPlatformerBody(size, jumpScale, pos) {

    if (pfBody)
      pfBody.remove();

    if (pnode)
      pnode.dead = true;

    var offset = V.mul(size, [-.5, .5]);
    pfBody = platformer.body(
      ng, pos, assets.envMat, walkSpeed,
      function() { return new p2.Box({width:size[0], height:size[1]}); },
      offset
    );

    pnode = pfBody.pnode;

    pnode.position.bind(posCell.set);
    pnode.isPlayer = true;
    pnode.object = player;

    var grav = ng.physics.world.gravity;
    grav = [grav[0], grav[1]];
    jumpVel = util.findJumpVel(jumpHeight * jumpScale, grav, pnode.body.mass);

    platformer.airControl(ng, pnode, dpad, {
      maxSpeed: walkSpeed,
      force: airControl
    });
  }

  initPlatformerBody([32-2, 48-2], 1, pos);



  var facing = frp.signal.event(1, dpad.map('0').filter());
  player.facing = facing;

  // Graphics

  var frames = [1, 2, 3, 2].map(function(x) {
    return graphics.image(
      assets.tiles,
      [16, 16],
      R.fromSize([32*x, 32*34], [32, 64])
    );
  });

  var walk = util.anim(ng, frames, .10);
  var stand = frp.signal.constant(frames[0]);

  function branch(a, b, c) {
    return a.flatMap(function(x) {
      return x ? b : c;
    });
  }

  var aframe = branch(dpad.map('0'), walk, stand);

  var frame = aframe;


  player.growScale = frp.cell(1);

  var graphic = frame;

  graphic = graphics.transform(
    graphic,
    player.growScale.map(function(x) { return T.scale([x, x]); })
  );

  var gtrv = frp.signal.combine({
    facing: facing
  }).map(function(x) {
    return T.prs([-16, 0], 0, [x.facing, 1]);
  });
  graphic = graphics.transform(graphic, gtrv);


  graphic = graphics.list([
    graphic,
    grow.overlayGraphic(player)
  ]);

  graphic = graphics.transform(
    graphic,
    posCell.map(T.pos)
  );

  player.graphic = graphic;

  // Jump


  controls.keydown('J').watch(function() {
    if (pfBody.grounded.value) {
      var v = pnode.body.velocity;
      pnode.body.velocity = V.add(v, jumpVel);
    }
  });

  player.growDuration = 5;

  player.onGrowStart = function() {
    initPlatformerBody([64, 96], 2, V.add(posCell.value, [0, -48]));
  };

  player.onShrinkStart = function() {
    initPlatformerBody([32, 48], 1, posCell.value);
  };

  player.pos = posCell;
  player.type = 'player';

  return player;

};

